/**
  * @file SignalRelayPIC32.c
  * @brief This module contains the implementation of the signal relay hardware
  * functions.
  *
  * @author Stefan Popovic
  *
  * @date 14/10/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: P32MX795F512L
  * Dev.Board: EasyPIC Fusion v7
  * External Clock 80.0000 MHz
  * Ext. Modules: Signal Relay Click Board
  * SW: MikroC PRO for PIC32 v3.6.0
  *
  * NOTES:
    - Signal Relay click board is essentially board with 4 5VDC Signal Relays
    - Place Signal Relay click board on mikroBUS slot 1
    - Pull-down PORTA pins 14 and 15 and PORTC pins 3 and 4. 
      Turn on Button Press Level switch SW10 for PORTA&C. (board specific)
    - RE 1: RB8
    - RE 2: RC1
    - RE 3: RC2
    - RE 4: RD0

  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/


  /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/


  /**************************************************************************
  * Module Typedefs
  **************************************************************************/


   /**************************************************************************
   * Includes
   **************************************************************************/
   #include "built_in.h"
   #include "recources.h"
   #include <stdbool.h>

  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/

  char txt[24];
  bool oldA15;
  bool oldA14;
  bool oldC4;
  bool oldC3;
    
  // TFT module connections
  char TFT_DataPort at LATE;
  sbit TFT_RST at LATD7_bit;
  sbit TFT_BLED at LATD2_bit;
  sbit TFT_RS at LATD9_bit;
  sbit TFT_CS at LATD10_bit;
  sbit TFT_RD at LATD5_bit;
  sbit TFT_WR at LATD4_bit;
  char TFT_DataPort_Direction at TRISE;
  sbit TFT_RST_Direction at TRISD7_bit;
  sbit TFT_BLED_Direction at TRISD2_bit;
  sbit TFT_RS_Direction at TRISD9_bit;
  sbit TFT_CS_Direction at TRISD10_bit;
  sbit TFT_RD_Direction at TRISD5_bit;
  sbit TFT_WR_Direction at TRISD4_bit;
  // End TFT module connections
  
   void Display_Init()
   {
    TFT_BLED_Direction = 0;
    TFT_Init_ILI9341_8bit(320, 240);
    TFT_BLED = 1;
   }
 
   // Draw Frame on TFT module
   void DrawFrame()
   {
    TFT_Fill_Screen(CL_WHITE);
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1,CL_BLACK, 0, 0, CL_BLACK, CL_BLACK);
    TFT_Line(20, 220, 300, 220);
    TFT_Line(5,  46, 315,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Signal Relay  Click  Board  Example", 5, 14);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyPIC Fusion v7 for P32", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);

    TFT_Write_Text("RELAY 1:", 10, 90 );
    TFT_Write_Text("RELAY 2:", 170, 90 );
    TFT_Write_Text("RELAY 3:", 10, 150);
    TFT_Write_Text("RELAY 4:", 170, 150 );
    
    TFT_Set_Pen(CL_RED, 5);
    TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
    
    // Switch for RELAY 1
    TFT_Line(80, 100, 100, 100);
    TFT_Line(130, 100, 150, 100);
    TFT_Line(100, 100, 130, 80);           // OPEN switch
    
    // Switch for RELAY 2
    TFT_Line(240, 100, 260, 100);
    TFT_Line(290, 100, 310, 100);
    TFT_Line(260, 100, 290, 80);           // OPEN switch
    
    // Switch for RELAY 3
    TFT_Line(80, 155, 100, 155);
    TFT_Line(130, 155, 150, 155);
    TFT_Line(100, 155, 130, 135);          // OPEN switch
    
    // Switch for RELAY 4
    TFT_Line(240, 155, 260, 155);
    TFT_Line(290, 155, 310, 155);
    TFT_Line(260, 155, 290, 135);          // OPEN switch
   }
  
    void main()
    {
      Display_Init();
      DrawFrame();
      
      oldA15 = false;
      oldA14= false;
      oldC4 = false;
      oldC3 = false;
      
      AD1PCFG = 0xFFFF;                    // Configure AN pins as digital I/O
      JTAGEN_bit = 0;                      // Disable JTAG
      TRISB8_bit = 0;                      // Set PORTB.8 pin as output
      TRISC1_bit = 0;                      // Set PORTC.1 pin as output
      TRISC2_bit = 0;                      // Set PORTC.2 pin as output
      TRISD0_bit = 0;                      // Set PORTD.0 pin as output
      
      TRISA15_bit = 1;                     // Set PORTA.15 pin as input
      TRISA14_bit = 1;                     // Set PORTA.14 pin as input
      TRISC4_bit = 1;                      // Set PORTC.4 pin as input
      TRISC3_bit = 1;                      // Set PORTC.3 pin as input

      LATB8_bit = 0;                       // Set PORTB.8 pin value to 0
      LATC1_bit = 0;                       // Set PORTC.1 pin value to 0
      LATC2_bit = 0;                       // Set PORTC.2 pin value to 0
      LATD0_bit = 0;                       // Set PORTD.0 pin value to 0
      
      while(1) 
      {
          
          // Relay 1
          if (Button(&PORTA, 15, 1, 1))    // detect logical one on PORTA15 pin
            {

              if (!oldA15)                 // detect zero-to-one transition
              {
                Delay_ms(200);

                //marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // OPEN switch
                TFT_Line(100, 100, 130, 80);     
              
                //showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
                
                // CLOSED switch
                TFT_Line(100, 100, 130, 100);     
                
                oldA15 = 1;
                LATB8_bit = 1;               
              }
              else                         // detect one-to-zero transition
              {
                Delay_ms(200);

                //marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // CLOSED switch
                TFT_Line(100, 100, 130, 100);     
                
                //showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

                 // OPEN switch
                TFT_Line(100, 100, 130, 80);    

                oldA15 = 0;
                LATB8_bit = 0;
              }
            }
          
          // Relay 2
          if (Button(&PORTC, 4, 1, 1))     // detect logical one on PORTC4 pin
            {

              if (!oldC4)                  // detect zero-to-one transition
              {
                Delay_ms(200);

                //marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // OPEN switch
                TFT_Line(260, 100, 290, 80);    

                //showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

                // CLOSED switch
                TFT_Line(260, 100, 290, 100); 

                oldC4 = 1;
                LATC1_bit = 1;

              }
              else                         // detect one-to-zero transition
              {
                Delay_ms(200);

                //marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
                
                // CLOSED switch
                TFT_Line(260, 100, 290, 100); 

                //showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
                
                 // OPEN switch
                TFT_Line(260, 100, 290, 80);   

                oldC4 = 0;
                LATC1_bit = 0;
              }
            }
          
          // Relay 3
          if (Button(&PORTA, 14, 1, 1))    // detect logical one on PORTA14 pin
            {

              if (!oldA14)                 // detect zero-to-one transition
              {
                Delay_ms(200);

                //marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
                
                // OPEN switch
                TFT_Line(100, 155, 130, 135); 

                //showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
                
                // CLOSED switch
                TFT_Line(100, 155, 130, 155); 

                oldA14 = 1;
                LATC2_bit = 1;

              }
              else                         // detect one-to-zero transition
              {
                Delay_ms(200);

                //marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                 // CLOSED switch
                TFT_Line(100, 155, 130, 155);

                //showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
                TFT_Line(100, 155, 130, 135); // OPEN switch

                oldA14 = 0;
                LATC2_bit = 0;
              }
            }
          
          // Relay 4
          if (Button(&PORTC, 3, 1, 1))     // detect logical one on PORTA14 pin
            {

              if (!oldC3)                  // detect zero-to-one transition
              {
                Delay_ms(200);

                //marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // OPEN switch
                TFT_Line(260, 155, 290, 135); 

                //showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

                // CLOSED switch
                TFT_Line(260, 155, 290, 155); 

                oldC3 = 1;
                LATD0_bit = 1;
              }
              else                         // detect one-to-zero transition
              {
                Delay_ms(200);

                //marking
                TFT_Set_Pen(CL_WHITE, 5);
                TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);

                // CLOSED switch
                TFT_Line(260, 155, 290, 155); 

                //showing
                TFT_Set_Pen(CL_RED, 5);
                TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

                 // OPEN switch
                TFT_Line(260, 155, 290, 135);

                oldC3 = 0;
                LATD0_bit = 0;
              }
            }         
      }
    }